# This Python file uses the following encoding: utf-8

# ARandR -- Another XRandR GUI
# Copyright (C) 2008 -- 2011 chrysn <chrysn@fsfe.org>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import gettext
gettext.install('arandr')

__version__ = '0.1.7.1'
PROGRAMNAME = _(u'ARandR Screen Layout Editor')
## translators, please translate in the style of "Another XRandR GUI
## (ein weiteres GUI für XRandR)" so users get both the explanation of
## the acronym and a localized version.
PROGRAMDESCRIPTION = _(u'Another XRandR GUI')
COPYRIGHT = u'© chrysn 2008 – 2013, Себастьян Gli ţa Κατινα 2011'

TRANSLATORS = [
        'chrysn <chrysn@fsfe.org>',
        'Quizzlo <paolone.marco@gmail.com>',
        'Phantom X <megaphantomx@bol.com.br>',
        'Joe Hansen <joedalton2@yahoo.dk>',
        'gundachandru <gundachandru@gmail.com>',
        'HsH <hsh@runtu.org>',
        'Clément Démoulins <clement@archivel.fr>',
        'RooTer <rooter@kyberian.net>',
        'Mohammad Alhargan <malham1@gmail.com>',
        'Mehmet Gülmen <memetgulmen@gmail.com>',
        'Ricardo A. Hermosilla Carrillo <ra.hermosillac@gmail.com>',
        'el_libre <el.libre@gmail.com>',
        'aboodilankaboot <shiningmoon25@gmail.com>',
        'Carezero <carezero@qq.com>',
        'wimfeijen <wimfeijen@gmail.com>',
        'Slavko <linux@slavino.sk>',
        'Alireza Savand <agahia.com@gmail.com>',
        'Bruno_Patri <bruno.patri@gmail.com>',
        'Себастьян Gli ţa Κατινα <gseba@users.sourceforge.net>',
        'Belvar <glasbarg@gmail.com>',
        'Algimantas Margevičius <margevicius.algimantas@gmail.com>',
        'Miguel Anxo Bouzada <mbouzada@gmail.com>',
        'o-157',
        'Ingemar Karlsson <ingemar@ingk.se>',
        'Semsudin Abdic <abdic88@gmail.com>',
        'Rax Garfield',
        'Tamás Nagy <kisagy@gmail.com>',
        'Mantas Kriaučiūnas <mantas@akl.lt>',
        'Dimitris Giouroukis',
        'ParkJS <HeavensBus@gmail.com>',
        ]
